"""

    distutilazy.test
    -----------------

    command classes to help run tests

    :license: MIT, see LICENSE for more details.

"""

from __future__ import absolute_import

import os
import sys
import glob
import importlib
import unittest
from distutils.core import Command

__version__ = "0.1.0"

class run_tests(Command):
    description = """Run test suite"""
    user_options = [("root=", "r", "path to tests suite dir"),
                    ("pattern=", "p", "test file name pattern"),
                    ("verbosity=", "v", "verbosity level [1,2,3]"),
                    ("files=", None, "run specified test files (comma separated)")]

    def initialize_options(self):
        self.root = os.path.join(os.getcwd(), 'tests')
        self.pattern = "test*.py"
        self.verbosity = 1
        self.files = None

    def finalize_options(self):
        if not os.path.exists(self.root):
            raise IOError("Failed to access root path " + self.root)
        verbosity = min(int(self.verbosity), 3)
        if verbosity < 1:
            self.verbosity = 1
        else:
            self.verbosity = verbosity
        if self.files:
            self.files = map(lambda name: name.strip(), self.files.split(','))

    def get_modules_from_files(self, files):
        modules = []
        for filename in files:
            dirname = os.path.dirname(filename)
            package_name = os.path.basename(dirname)
            if package_name:
                try:
                    self.announce("importing {0} as package ...".format(package_name))
                    package = importlib.import_module(package_name)
                    if hasattr(package, '__path__') and os.path.abspath(package.__path__[0]) != os.path.abspath(dirname):
                        raise ImportError("directory {1} is not a package to import".format(dirname))
                except ImportError as err:
                    self.announce("failed to import {0}. not a package. {1}".format(package_name, err))
                    sys.path.insert(0, dirname)
                    package_name = None
            modulename, _, extension = os.path.basename(filename).rpartition('.')
            if not modulename:
                self.announce("failed to find module name from filename '{0}'. skipping this file".format(filename))
                continue
            if package_name:
                modulename = '.' + modulename
            self.announce("importing module {0} from file {1} ...".format(modulename, filename))
            module = importlib.import_module(modulename, package=package_name)
            modules.append(module)
        return modules

    def find_test_modules_from_package_path(self, package_path):
        """Check if the path is a package, and if it introduces modules"""
        package_dir = os.path.dirname(package_path)
        package_name = os.path.basename(package_path)
        if package_dir:
            sys.path.insert(0, package_dir)
        try:
            package = importlib.import_module(package_name)
            if package and hasattr(package, '__all__'):
                modules = []
                for module_name in package.__all__:
                    module = importlib.import_module('{0}.{1}'.format(package_name, module_name))
                    modules.append(module)
                return modules
        except ImportError as exp:
            pass
        return []

    def find_test_modules_from_test_files(self, root, pattern):
        """Return list of test modules from the path whose filename matches the pattern"""
        test_files = glob.glob(os.path.join(root, pattern))
        if not test_files:
            return []
        package_name = os.path.basename(root)
        try:
            self.announce("importing {0} as package ...".format(package_name))
            package = importlib.import_module(package_name)
            if hasattr(package, '__path__') and os.path.abspath(package.__path__[0]) != os.path.abspath(root):
                raise ImportError("directory {1} is not a package to import".format(root))
        except ImportError as err:
            self.announce("failed to import {0}. not a package. {1}".format(package_name, err))
            sys.path.insert(0, root)
            package_name = None
        modules = []
        for filename in test_files:
            modulename, _, extension = os.path.basename(filename).rpartition('.')
            if not modulename:
                self.announce("failed to find module name from filename '{0}'. skipping this test".format(filename))
                continue
            if package_name:
                modulename = '.' + modulename
            self.announce("importing module {0} from file {1} ...".format(modulename, filename))
            module = importlib.import_module(modulename, package=package_name)
            modules.append(module)
        return modules

    def test_suite_for_modules(self, modules):
        suite = unittest.TestSuite()
        testLoader = unittest.defaultTestLoader
        for module in modules:
            module_tests = testLoader.loadTestsFromModule(module)
            suite.addTests(module_tests)
        return suite

    def get_test_runner(self):
        return unittest.TextTestRunner(verbosity=self.verbosity)

    def run(self):
        modules = None
        if self.files:
            modules = self.get_modules_from_files(self.files)
        else:
            self.announce("searching for test package modules ...")
            modules = self.find_test_modules_from_package_path(self.root)
            if not modules:
                self.announce("searching for test files ...")
                modules = self.find_test_modules_from_test_files(self.root, self.pattern)
        if not modules:
            self.announce("found no test files")
            return False
        suite = self.test_suite_for_modules(modules)
        runner = self.get_test_runner()
        self.announce("running tests ...")
        runner.run(suite)



"""
    distutilazy.tests
    -----------------

    Tests for distutilazy

    :license: MIT, see LICENSE for more details.
"""

from os import path
import glob

test_modules = [path.splitext(path.basename(filename))[0] for filename in glob.glob(path.join(path.dirname(__file__), 'test*.py'))]
test_modules.sort()
__all__ = test_modules



"""
    distutilazy.tests.test_pyinstaller
    ----------------------------------

    Tests for distutilazy.pyinstaller module.

    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import sys
import os
import unittest
import re

from .setup_test_env import *
from distutilazy import pyinstaller
from distutils.dist import Distribution

class TestPyinstaller(unittest.TestCase):

    def test_finalize_opts(self):
        dist = Distribution()
        pi = pyinstaller.bdist_pyinstaller(dist)
        pi.target = "fake.py"
        pi.finalize_options()
        self.assertTrue( re.match(".+", pi.name) )
        self.assertTrue(pi.pyinstaller_opts)

    def test_clean_all(self):
        dist = Distribution()
        cl = pyinstaller.clean_all(dist)
        cl.finalize_options()
        paths = cl.get_extra_paths()
        self.assertTrue(paths)
        spec = paths.pop()
        self.assertTrue( re.match("\S+\.spec", spec) )



"""
    distutilazy.tests.test_test
    ----------------------------

    Tests for distutilazy.test module

    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import sys
import os
from os import path
import shutil
import unittest

from .setup_test_env import TEST_DIR
from distutilazy import test
from distutils.dist import Distribution

# if running in a cached compiled python file
# assume filename as python source file.
# when tests are running together, test_clean
# removes pyc files. but .py file will be available
if __file__[-1].lower() == 'c':
    __file__ = __file__[:-1]

class TestTest(unittest.TestCase):
    def _get_module_filenames(self, modules):
        return map(lambda m: path.basename(m.__file__), modules)

    def test_find_modules_from_package_path(self):
        dist = Distribution()
        test_ = test.run_tests(dist)
        test_.finalize_options()
        here = path.dirname(__file__)
        filename = path.basename(__file__)
        modules = test_.find_test_modules_from_package_path(here)
        self.assertIn(filename, self._get_module_filenames(modules))

    def test_get_modules_from_files(self):
        dist = Distribution()
        test_ = test.run_tests(dist)
        test_.finalize_options()
        self.assertEqual([], test_.get_modules_from_files(['none_existing_file']))
        modules = test_.get_modules_from_files([__file__])
        self.assertEqual(1, len(modules))
        self.assertEqual(path.basename(__file__), path.basename(modules.pop().__file__))

    def test_find_modules_from_files(self):
        dist = Distribution()
        test_ = test.run_tests(dist)
        test_.finalize_options()
        here = path.dirname(__file__)
        filename = path.basename(__file__)
        modules = test_.find_test_modules_from_test_files(here, 'none_exiting_pattern')
        self.assertEqual([], modules)
        modules = test_.find_test_modules_from_test_files(here, filename)
        self.assertEqual(1, len(modules))
        self.assertEqual(filename, path.basename(modules.pop().__file__))
        modules = test_.find_test_modules_from_test_files(here, 'test_*')
        self.assertIn(filename, self._get_module_filenames(modules))

    def test_test_suite_for_modules(self):
        dist = Distribution()
        test_ = test.run_tests(dist)
        test_.finalize_options()
        suite = test_.test_suite_for_modules([])
        self.assertIsInstance(suite, unittest.TestSuite)

    def test_get_test_runner(self):
        dist = Distribution()
        test_ = test.run_tests(dist)
        test_.finalize_options()
        runner = test_.get_test_runner()
        self.assertTrue(hasattr(runner, 'run'))
        self.assertTrue(hasattr(runner.run, '__call__'))




"""

    distutilazy.pyinstaller
    -----------------------

    command classes to call pyinstaller

    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import os
import platform
import subprocess
from distutils.core import Command
from distutils.errors import DistutilsOptionError

from . import clean

__version__ = "0.2.0"

is_windows = platform.system().upper() == "WINDOWS"
path_separator = is_windows and ';' or ':'

class bdist_pyinstaller(Command):
    """Distutils command to run PyInstaller with configured defaults"""

    description = """Run PyInstaller to compile standalone binary executables"""

    user_options = [
        ("target=", 't', "Taget Python app to bundle"),
        ("pyinstaller-path=", None, "Path to pyinstaller executable"),
        ("name=", 'n', "Name of the bundled app"),
        ("icon=", 'i', "Path to icon resource"),
        ("windowed", 'w', "Windowed app, no console for stdio"),
        ("clean", None, "Clean cached and temp files before build"),
        ("hidden-imports=", 'I', "comma separated list of extra modules to be imported"),
        ("paths=", 'p', "extra paths to search for modules separated by '{0}'".format(path_separator)),
    ]

    boolean_options = ["windowed", "clean"]

    def initialize_options(self):
        self.pyinstaller_opts = []
        self.imports = []
        self.syspaths = []
        self.target = None
        self.pyinstaller_path = None
        self.name = None
        self.icon = None
        self.windowed = None
        self.clean = None
        self.hidden_imports = None
        self.paths = None

    def default_pyinstaller_opts(self):
        """Return default options for PyInstaller.
        Use this method to customize the command for separate projects

        :return: list of options
        """
        return ["--onefile"]

    def default_imports(self):
        """Return list of explicit imports.
        Use this method to customize the extra modules for separate projects

        :return: list of module names to be imported
        """
        return []

    def default_paths(self):
        """Return list of paths to append to sys.path while importing modules.
        Use this method to customize the extra paths for separate projects

        :return: list of paths
        """
        return []

    def finalize_options(self):
        if self.pyinstaller_path:
            self.pyinstaller_path = os.path.abspath(self.pyinstaller_path)
            if not os.path.exists(self.pyinstaller_path):
                raise DistutilsOptionError("failed to find pyinstaller from " + self.pyinstaller_path)
        self.pyinstaller_opts.extend( self.default_pyinstaller_opts() )
        if not self.name:
            self.name = self.distribution.metadata.get_name()
        if self.clean:
            self.pyinstaller_opts.append("--clean")
        if self.windowed:
            self.pyinstaller_opts.append("--windowed")
        if self.icon:
            self.pyinstaller_opts.append("--icon=" + self.icon)
        if not is_windows:
            self.pyinstaller_opts.append("--strip")

        self.imports.extend( self.default_imports() )
        if self.hidden_imports:
            self.imports.extend( [ i.strip() for i in self.hidden_imports.split(',') if i.strip()] )
        for mod in self.imports:
            self.pyinstaller_opts.append("--hidden-import=" + mod)

        self.syspaths.extend( self.default_paths() )
        if self.paths:
            self.syspaths.extend( [p for p in self.paths.split(path_separator)] )
        for path in self.syspaths:
            self.pyinstaller_opts.append("--paths=" + path)
        self.pyinstaller_opts.append("--name=" + self.name)

    def run(self):
        if not self.target:
            raise DistutilsOptionError("no target app is specified to bundle")
        pi = self.pyinstaller_path or "pyinstaller"
        args = self.pyinstaller_opts
        args.append(self.target)
        args.insert(0, pi)
        self.announce("running " + ' '.join(args))
        code = subprocess.call(args)
        return code

class clean_all(clean.clean_all):
    """Distutils command to clean all temporary files, compiled Python files, PyInstaller temp files and spec."""

    user_options = [
        ("keep-build", None, "do not clean build direcotry"),
        ("keep-dist", None, "do not clean dist direcotry"),
        ("keep-egginfo", None, "do not clean egg info direcotry"),
        ("keep-extra", None, "do not clean extra files"),
        ("name", None, "name of the bundled app"),
    ]

    def initialize_options(self):
        clean.clean_all.initialize_options(self)
        self.name = None

    def finalize_options(self):
        clean.clean_all.finalize_options(self)
        if not self.name:
            self.name = self.distribution.metadata.get_name()

    def get_extra_paths(self):
        """Return list of extra files/directories to be removed"""
        return [self.name + ".spec"]



"""
    distutilazy.command.test
    ------------------------

    Command to run unit tests

    :license: MIT, see LICENSE for more details.
"""

import os
import sys

base_dir = os.path.abspath(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
if not base_dir in sys.path:
    if len(sys.path):
        sys.path.insert(1, base_dir)
    else:
        sys.path.append(base_dir)

import distutilazy.test

class test(distutilazy.test.run_tests):
    pass



"""
    distutilazy.tests.test_util
    -----------------

    Test distutilazy.util module.

    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import os
from os import path
import sys
import unittest

from .setup_test_env import TEST_DIR
from distutilazy import util

# if running in a cached compiled python file
# assume filename as python source file.
# when tests are running together, test_clean
# removes pyc files. but .py file will be available
if __file__[-1].lower() == 'c':
    __file__ = __file__[:-1]

class TestUtil(unittest.TestCase):

    def test_util_find_files(self):
        me = os.path.realpath(__file__)
        files = util.find_files(TEST_DIR, "test_util.py*")
        self.assertTrue(me in files)
        files = util.find_files(TEST_DIR, "not_existing_file.py")
        self.assertEqual(files, [])

    def test_util_find_directories(self):
        found = util.find_directories(path.dirname(TEST_DIR), "tes*")
        self.assertTrue(TEST_DIR in found)
        found = util.find_directories(TEST_DIR, "not_existing_dir")
        self.assertEqual(found, [])



"""

    distutilazy.command
    -----------

    Extra commands for setup.py using classes provided by distutilazy

    :license: MIT, see LICENSE for more details.
"""

import os
import sys

__all__ = ["clean_pyc", "clean_all", "bdist_pyinstaller", "test"]

base_dir = os.path.abspath(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
if not base_dir in sys.path:
    if len(sys.path):
        sys.path.insert(1, base_dir)
    else:
        sys.path.append(base_dir)





"""
    distutilazy.tests.test_clean
    ----------------------------

    Tests for distutilazy.clean module

    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import sys
import os
from os import path
import shutil
import unittest

from .setup_test_env import TEST_DIR
from distutilazy import clean
from distutils.dist import Distribution

class TestClean(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.test_cache_dir = path.join(TEST_DIR, '_test_py_cache_')
        if path.exists(cls.test_cache_dir):
            raise Exception(
                    "Test python cache directory exsits in {0}. Please remove this path".format(
                        cls.test_cache_dir
                        )
                    )
        else:
            os.mkdir(cls.test_cache_dir)

    @classmethod
    def tearDownAfter(cls):
        if path.exists(cls.test_cache_dir):
            shutil.rmtree(cls.test_cache_dir, True)

    def test_clean_all(self):
        dist = Distribution()
        dist.metadata.name = "testdist"
        cl = clean.clean_all(dist)
        cl.finalize_options()
        self.assertEqual(cl.get_egginfo_dir(), "testdist.egg-info")
        targets = ["build", "dist", "egginfo", "extra"]
        bad_calls = []
        good_calls = []
        good_calls_should_be = 0
        for target in targets:
            cl = clean.clean_all(dist)
            cl.finalize_options()
            cl.dry_run = True
            setattr(cl, "keep_%s" % target, True)
            setattr(cl, "clean_%s" % target, lambda self: bad_calls.append(targt))
            other_targets = [t for t in targets if t != target]
            for ot in other_targets:
                good_calls_should_be += 1
                setattr(cl, "clean_%s" % ot, lambda self=None: good_calls.append(ot))
            cl.run()
        self.assertEqual(bad_calls, [])
        self.assertEqual(len(good_calls), good_calls_should_be)

    def test_clean_pyc(self):
        dist = Distribution()
        cl = clean.clean_pyc(dist)
        cl.extensions = "ppyycc, ppyyoo"
        cl.finalize_options()
        self.assertEqual(cl.extensions, ["ppyycc", "ppyyoo"])
        self.assertEqual(cl.find_compiled_files(), [])

    def test_clean_py_cache_dirs(self):
        dist = Distribution()
        cl = clean.clean_pyc(dist)
        cl.directories = "_test_py_cache_"
        cl.finalize_options()
        self.assertEqual(cl.directories, ["_test_py_cache_"])
        self.assertEqual(cl.find_cache_directories(), [self.__class__.test_cache_dir])
        cl.run()
        self.assertFalse(path.exists(self.__class__.test_cache_dir))

    def test_clean_py_cache_dirs_finds_nothing(self):
        dist = Distribution()
        cl = clean.clean_pyc(dist)
        cl.extensions = "ppyycc, ppyyoo"
        cl.directories = "not_exist, and_not_found"
        cl.finalize_options()
        self.assertEqual(cl.directories, ["not_exist", "and_not_found"])
        self.assertEqual(cl.find_cache_directories(), [])



"""

    distutilazy.command.clean_pyc
    -----------------------------

    Command to clean compiled python files

    :license: MIT, see LICENSE for more details.
"""

import os
import sys

base_dir = os.path.abspath(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
if not base_dir in sys.path:
    if len(sys.path):
        sys.path.insert(1, base_dir)
    else:
        sys.path.append(base_dir)

import distutilazy.clean

class clean_pyc(distutilazy.clean.clean_pyc):
    pass



"""
    distutilazy
    -----------

    Extra distutils command classes.

    :license: MIT, see LICENSE for more details.
"""

import os
import sys

__version__ = '0.3.1'
__all__ = ['clean', 'pyinstaller', 'command']

base_dir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
if not base_dir in sys.path:
    if len(sys.path):
        sys.path.insert(1, base_dir)
    else:
        sys.path.append(base_dir)



"""

    distutilazy.clean
    -----------------

    command classes to help clean temporary files

    :license: MIT, see LICENSE for more details.

"""

from __future__ import absolute_import

import os
import shutil
from distutils import log
from distutils.core import Command
from distutils.command import clean

from . import util

__version__ = "0.4.0"

class clean_pyc(Command):
    description = """Clean root dir from complied python files"""
    user_options = [("root=", "r", "path to root dir")]

    def initialize_options(self):
        self.root = os.getcwd()
        self.extensions = "pyc,pyo,pyd"
        self.directories = "__pycache__,"

    def finalize_options(self):
        if not os.path.exists(self.root):
            raise IOError("Failed to access root path " + self.root)
        self.extensions = [ext.strip() for ext in self.extensions.split(',')]
        self.directories = [dirname.strip() for dirname in self.directories.split(',')]

    def find_compiled_files(self):
        """Find compiled Python files recursively in the root path

        :return: list of absolute file paths
        """
        files = []
        for ext in self.extensions:
            extfiles = util.find_files(self.root, "*." + ext)
            log.debug("found {0} .{1} files in {2}".format(len(extfiles), ext, self.root))
            files.extend(extfiles)
            del extfiles
        self.announce("found {0} compiled python files in {1}".format(len(files), self.root))
        return files

    def find_cache_directories(self):
        directories = []
        for dirname in self.directories:
            dirs = util.find_directories(self.root, dirname)
            log.debug("found {0} directories in {1}".format(len(dirs), self.root))
            directories.extend(dirs)
            del dirs
        self.announce("found {0} python cache directories in {1}".format(len(directories), self.root))
        return directories

    def _clean_file(self, filename):
        """Clean a file if exists"""
        if not os.path.exists(filename):
            return
        self.announce("removing " + filename)
        if not self.dry_run:
            os.remove(filename)

    def _clean_directory(self, dirname):
        """Clean a directory if exists"""
        if not os.path.exists(dirname):
            return
        self.announce("removing directory {0} and all it's contents".format(dirname))
        if not self.dry_run:
            shutil.rmtree(dirname, True)

    def run(self):
        dirs = self.find_cache_directories()
        if dirs:
            self.announce("cleaning python cache directories in {0} ...".format(self.root))
            if not self.dry_run:
                for dirname in dirs:
                    self._clean_directory(dirname)

        files = self.find_compiled_files()
        if files:
            self.announce("cleaning compiled python files in {0} ...".format(self.root))
            if not self.dry_run:
                for filename in files:
                    self._clean_file(filename)

class clean_all(clean.clean, clean_pyc):
    description = """Clean root dir from temporary files, complied files, etc."""
    user_options = [
        ("keep-build", None, "do not clean build direcotry"),
        ("keep-dist", None, "do not clean dist direcotry"),
        ("keep-egginfo", None, "do not clean egg info direcotry"),
        ("keep-extra", None, "do not clean extra files"),
    ]

    boolean_options = ["keep-build", "keep-dist", "keep-egginfo", "keep-extra"]

    def initialize_options(self):
        clean.clean.initialize_options(self)
        clean_pyc.initialize_options(self)
        self.keep_build = None
        self.keep_dist = None
        self.keep_egginfo = None
        self.keep_extra = None

    def finalize_options(self):
        clean.clean.finalize_options(self)
        clean_pyc.finalize_options(self)
        self.all = True

    def get_egginfo_dir(self):
        return self.distribution.metadata.get_name() + ".egg-info"

    def get_extra_paths(self):
        """Return list of extra files/directories to be removed"""
        return []

    def clean_egginfo(self):
        """Clean .egginfo directory"""
        dirname = os.path.join(self.root, self.get_egginfo_dir())
        self._clean_directory(dirname)

    def clean_dist(self):
        self._clean_directory(os.path.join(self.root, "dist"))

    def clean_build(self):
        self._clean_directory(os.path.join(self.root, "build"))

    def clean_extra(self):
        """Clean extra files/directories specified by get_extra_paths()"""
        extra_paths = self.get_extra_paths()
        for path in extra_paths:
            if not os.path.exists(path):
                continue
            if os.path.isdir(path):
                self._clean_directory(path)
            else:
                self._clean_file(path)

    def run(self):
        clean.clean.run(self)
        clean_pyc.run(self)
        if not self.keep_build:
            self.clean_build()
        if not self.keep_egginfo:
            self.clean_egginfo()
        if not self.keep_dist:
            self.clean_dist()
        if not self.keep_extra:
            self.clean_extra()



"""

    distutilazy.tests.setup_test_env
    -----------------

    Sets up the test environment and provides methods to access the test data

    :license: MIT, see LICENSE for more details.

"""

import os
import sys

__version__ = "0.1.0"
__all__ = ['setupenv', 'TEST_DIR']

TEST_DIR = os.path.realpath(os.path.dirname(__file__))

def setupenv():
    try:
        sys.path.index(TEST_DIR)
    except ValueError:
        sys.path.insert(0, TEST_DIR)
    base_path = os.path.realpath(os.path.join(TEST_DIR, '..'))
    try:
        sys.path.index(base_path)
    except ValueError:
        sys.path.insert(0, base_path)
    import distutilazy

setupenv()
